/*
================================================================================
 File: converter.c
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       USBD RS232C converter
 Compiler:      ANSI C
 Hardware:
 Doc:
--------------------------------------------------------------------------------
 Includes:
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostpnione jest do celw pogldowych,
                nie jest przetestowane pod ktem bezpieczestwa ani spenienia
                jakichkolwiek norm wymaganych aby urzdzenie uywajce go
                mogo by dopuszczone do uytku, dlatego uywasz go
                na wasn odpowiedzialno a jego autor nie ponosi adnego
                typu odpowiedzialnoci za jakiekolwiek skutki jego uycia.
                Jeeli nie zgadzasz si z powyszym warunkiem prosz
                o nieuywanie tego oprogramowania.
================================================================================
*/

#include "converter.h"


// -----------------------------------------------------------------------------
//  converter objects
// -----------------------------------------------------------------------------

/**
 * USBDC
 * struktura opisujaca konfiguracje
 * musi byc utworzona na stale poniewaz jest uzywana
 * caly czas podczas uzywania USBD.
 */
static USBDC_params usbdc;
static uint8_t usbd_conv_descriptor[256];
uint8_t trash[USBDEP_FS_BULK_MAX_POSSIBLE_PACKET_SIZE];



static converter_params *conv_usart1;
static converter_params *conv_usart2;
static converter_params *conv_usart3;


#if(USE_VCOM1)
static converter_params vcom1_params;
#endif
#if(USE_VCOM2)
static converter_params vcom2_params;
#endif




/**
 * Manufacturer_String_Descriptor
 * String - jest to jeden z elementow USBD stosowany opcjonalnie
 */
static const uint16_t Manufacturer_String_Descriptor[] = {
	0x0328,    			// bLength, bDescriptorType: string descriptor
	'p',
	'i',
	'o',
	't',
	'r',
	'e',
	'k',
	'l',
	'c',
	'6',
	'0',
	' ',
	'K',
	'N',
	'E',
	' ',
	'A',
	'G',
	'H'
}; /* Manufacturer_String_Descriptor */


/**
 * Product_String_Descriptor
 * String - jest to jeden z elementow USBD stosowany opcjonalnie
 */
static const uint16_t Product_String_Descriptor[] = {
	0x0322,    			// bLength, bDescriptorType: string descriptor
	'V',
	'I',
	'R',
	'T',
	'U',
	'A',
	'L',
	' ',
	'C',
	'O',
	'M',
	' ',
	'P',
	'O',
	'R',
	'T'
}; /* Product_String_Descriptor */


/**
 * SerialNumber_String_Descriptor
 * String - jest to jeden z elementow USBD stosowany opcjonalnie
 */
static uint16_t SerialNumber_String_Descriptor[25];


/**
 * hex
 * przelicza liczbe w HEX (mlodsza polowka bajtu) na znak kodu ASCI
 */
static uint8_t hex (uint8_t l) {		//przelicza liczbe w HEX
	l &= 0xF;			// (mlodsza polowka bajtu)
	return("0123456789ABCDEF"[l]);
}


/**
 * SerialNumber_fill
 * wypelnia string dla USBD pobierajac unikalny numer procesora
 */
static void SerialNumber_fill(void) {
	SerialNumber_String_Descriptor[0] = 0x0332;

	uint8_t ID;
	uint8_t *p_ID = (uint8_t*)0x1FFFF7E8;
	uint8_t i;
	uint16_t *p_string = &SerialNumber_String_Descriptor[1];

	// w ponizszych liniach pobierany jest unikalny numer
	// ID procesora nadany przez producenta
	// i nastepnie umieszczany jest w stringu
	// SerialNumber bedacym parametrem USB device
	for(i = 0; i < 12; i++) {
		ID = p_ID[i];
		p_string[i << 1] = (uint16_t)hex(ID & 0xF);
		p_string[(i << 1) + 1] = (uint16_t)hex((ID >> 4) & 0xF);
	}
}






//	-----------------------------------------------------------------------------
/**
 *  vCOM_Rx
 *  Interrupt handler 
 *  cdc_vcom layer generate this function when endpoint receives some Rx data (from Host)
 */
//	-----------------------------------------------------------------------------
static uint32_t vCOM_Rx_handler (CDC_VCOM_params *vCOM, uint8_t* buffer, uint32_t received_size) {
	uint32_t ret;
	uint32_t size;
	converter_params *conv = vCOM->upper_layer_data.pvoid;

	size = BUF_GetFreeSpace(&(conv->buf.to_usart));
	if(size < received_size) {
		BUF_Read(&(conv->buf.to_usart), trash, received_size - size);
	}
	BUF_Write(&(conv->buf.to_usart), buffer, received_size);

	size = BUF_GetFreeSpace(&(conv->buf.to_usart));
	if(size >= USBDEP_FS_BULK_MAX_POSSIBLE_PACKET_SIZE) {
		ret = 0;
	}
	else {
		ret = 1;
	}
	USART_ITConfig(conv->USARTx, USART_IT_TXE, ENABLE);
	return(ret);
} /* vCOM_Rx */


//	-----------------------------------------------------------------------------
/**
 *  vCOM_Tx
 *  Interrupt handler 
 *  cdc_vcom layer generate this function after succesfully closed Tx (to Host) transfer
 */
 // -----------------------------------------------------------------------------
static uint32_t vCOM_Tx_handler (CDC_VCOM_params *vCOM, uint8_t* buffer, uint32_t available_size) {
	uint32_t size;
	converter_params *conv = vCOM->upper_layer_data.pvoid;

	size = BUF_GetCount(&(conv->buf.to_vcom));
	if(size) {
		if(size > available_size) {
			size = available_size;
		}
		BUF_Read(&(conv->buf.to_vcom), buffer, size);
	}
	return(size);
} /* vCOM_Tx */





static void USART_Proc(converter_params *conv, USART_TypeDef* USARTx) {
	uint8_t data;
	uint32_t size;

	// check status - if there was receive event
	if (USART_GetITStatus(USARTx, USART_IT_RXNE) != RESET) {
		// read data from register
		data = USARTx->DR;

		// if we have free space in buffer
		if(BUF_GetFreeSpace(&(conv->buf.to_vcom))) {
			// write received data to buffer
			BUF_Write(&(conv->buf.to_vcom), &data, 1);
		}//*/
	}

	// check status - if there was transmit event
	if (USART_GetITStatus(USARTx, USART_IT_TXE) != RESET) {
		// if we have some data in the buffer
		if(BUF_GetCount(&(conv->buf.to_usart))) {
			// read one byte
			BUF_Read(&(conv->buf.to_usart), &data, 1);

			// write byte to data register to transmit it
			USARTx->DR = data;
		}
		// we don't have any data in the buffer
		else {
			// disable transmitter
			USART_ITConfig(USARTx, USART_IT_TXE, DISABLE);
		}

		// get size of free space in the buffer
		size = BUF_GetFreeSpace(&(conv->buf.to_usart));
		// if free size is equal or bigger than size
		// of USB OUT Endpoint buffer
		if(size >= USBDEP_FS_BULK_MAX_POSSIBLE_PACKET_SIZE) {
			// enable receiving data through USB OUT Pipe
			CDC_VCOM_RxD_data_ready(&(conv->vcom));
		}
	}
}





void USART1_IRQHandler(void) {
	USART_Proc(conv_usart1, USART1);
}

void USART2_IRQHandler(void) {
	USART_Proc(conv_usart2, USART2);
}

void USART3_IRQHandler(void) {
	USART_Proc(conv_usart3, USART3);
}






static void configure_usart(uint8_t usart_num, uint32_t baudrate, uint16_t stop_bits, uint16_t patity, uint8_t enable_irq) {
	GPIO_InitTypeDef GPIO_InitStructure;
	USART_InitTypeDef USART_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;
	USART_TypeDef* USARTx;
	GPIO_TypeDef* GPIO_Rx_Port;
	GPIO_TypeDef* GPIO_Tx_Port;
	uint16_t GPIO_Rx_Pin;
	uint16_t GPIO_Tx_Pin;
	uint8_t NVIC_IRQChannel;


	// identify USART number, reset the peripheral and enable its clock
	{
		if(usart_num == 1) {
			USARTx = USART1;
			GPIO_Rx_Port = USART1_RX_PORT;
			GPIO_Tx_Port = USART1_TX_PORT;
			GPIO_Rx_Pin = USART1_RX_PIN;
			GPIO_Tx_Pin = USART1_TX_PIN;
			NVIC_IRQChannel = USART1_IRQn;

			// reset peripheral and enable its clock
			RCC_APB2PeriphResetCmd(RCC_APB2Periph_USART1, ENABLE);
			RCC_APB2PeriphResetCmd(RCC_APB2Periph_USART1, DISABLE);
			RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);
		}
		else if(usart_num == 2) {
			USARTx = USART2;
			GPIO_Rx_Port = USART2_RX_PORT;
			GPIO_Tx_Port = USART2_TX_PORT;
			GPIO_Rx_Pin = USART2_RX_PIN;
			GPIO_Tx_Pin = USART2_TX_PIN;
			NVIC_IRQChannel = USART2_IRQn;

			// reset peripheral and enable its clock
			RCC_APB1PeriphResetCmd(RCC_APB1Periph_USART2, ENABLE);
			RCC_APB1PeriphResetCmd(RCC_APB1Periph_USART2, DISABLE);
			RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);
		}
		else if(usart_num == 3) {
			USARTx = USART3;
			GPIO_Rx_Port = USART3_RX_PORT;
			GPIO_Tx_Port = USART3_TX_PORT;
			GPIO_Rx_Pin = USART3_RX_PIN;
			GPIO_Tx_Pin = USART3_TX_PIN;
			NVIC_IRQChannel = USART3_IRQn;

			// reset peripheral and enable its clock
			RCC_APB1PeriphResetCmd(RCC_APB1Periph_USART3, ENABLE);
			RCC_APB1PeriphResetCmd(RCC_APB1Periph_USART3, DISABLE);
			RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE);
		}
		else {
			return;
		}
	}


	// configure GPIO
	{
		GPIO_InitStructure.GPIO_Pin = GPIO_Tx_Pin;
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
		GPIO_Init(GPIO_Tx_Port, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = GPIO_Rx_Pin;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
		GPIO_Init(GPIO_Rx_Port, &GPIO_InitStructure);
	}


	// configure USART peripheral
	// and enable it
	{
		USART_InitStructure.USART_BaudRate = baudrate;
		USART_InitStructure.USART_WordLength = USART_WordLength_8b;
		USART_InitStructure.USART_StopBits = stop_bits;
		USART_InitStructure.USART_Parity = patity ;
		USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
		USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

		// Initialize registers
		USART_Init(USARTx, &USART_InitStructure);
		// Enable the USART
		USART_Cmd(USARTx, ENABLE);
	}


	// Enable the USART Interrupt
	if(enable_irq) {
		NVIC_InitStructure.NVIC_IRQChannel = NVIC_IRQChannel;
		NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
		NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
		NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
		NVIC_Init(&NVIC_InitStructure);

		// Configure the transmit and receive ISR
		USART_ITConfig(USARTx, USART_IT_TXE, ENABLE);
		USART_ITConfig(USARTx, USART_IT_RXNE, ENABLE);
	}
}



static void vCOM_change_params(CDC_VCOM_params *vCOM) {
	converter_params *conv;
	uint16_t stop_bits;
	uint16_t patity;

	if(CDC_VCOM_get_baudrate(vCOM) && CDC_VCOM_get_data_bits(vCOM)) {
		conv = vCOM->upper_layer_data.pvoid;

		switch(CDC_VCOM_get_stop_bits(vCOM)) {
			// 1 stop bit
			//case 0:  // -> default

			// 1.5 stop bits
			case 1:
				stop_bits = USART_StopBits_1_5;
				break;
			// 2 stop bits
			case 2:
				stop_bits = USART_StopBits_2;
				break;
			default:
				stop_bits = USART_StopBits_1;
				break;
		}

		switch(CDC_VCOM_get_partity(vCOM)) {
			// None
			//case 0:  // -> default
			// Odd
			case 1:
				patity = USART_Parity_Odd;
				break;
			// Odd
			case 2:
				patity = USART_Parity_Even;
				break;
			default:
				patity = USART_Parity_No;
				break;
		}

		configure_usart(conv->usart_num, CDC_VCOM_get_baudrate(vCOM), stop_bits, patity, 1);
	}
}


static void usbd_init(void) {
	USBD_device_descriptor *usbd_dev_desc;
	CCM_session_params ccm_session;


	// ----------------------------------------------------------------------
	// Inicjalizacja konfiguracji
	// ----------------------------------------------------------------------
	// otwarcie konfiguracji do automatycznego tworzenia
	// za pomoc biblioteki CCM
	CCM_open_config(USBD_STM32, &usbdc, &ccm_session);

	// dodanie wirtualnego portu szeregowego (vCOM)
	// (lub 2 vCOM-w) do tworzonej przez nas konfiguracji
#if(USE_VCOM1)
	CDC_VCOM_add_item(&(vcom1_params.vcom), &ccm_session);
#endif
#if(USE_VCOM2)
	CDC_VCOM_add_item(&(vcom2_params.vcom), &ccm_session);
#endif

	// zamkniecie sesji CCM i zainstalowanie konfiguracji -
	// po wyjsciu z tej funkcji mamy gotowy deskryptor konfiguracji
	// w tablicy usbd_conv_descriptor oraz wypelniona i zainstalowana
	// strukture usbdc.
	CCM_close_and_install_config(usbd_conv_descriptor, &ccm_session);



	// ----------------------------------------------------------------------
	// Zmiana pol deskryptora urzadzenia (Devce Descriptor)
	// ----------------------------------------------------------------------
	usbd_dev_desc = USBD_get_dev_descriptor_pointer(&usbdc);
	// tu wypelniamy sobie parametry vendorID i productID
	usbd_dev_desc->idVendor           = 0x0483;
	usbd_dev_desc->idProduct          = 0x5740;

	// jakiej wersji USB uzywamy
	usbd_dev_desc->bcdUSB            = 0x0110;

	// klasa urzadzenia od razu informuje,
	// ze jest to urzadzenie komunikacyjne
	usbd_dev_desc->bDeviceClass       = 0x02;



	// ----------------------------------------------------------------------
	// Elementy opcjonalne - stringi
	// ----------------------------------------------------------------------
	// zainstalowanie stringow
	USBDS_install((uint8_t*)Manufacturer_String_Descriptor, &(usbd_dev_desc->iManufacturer), &usbdc);
	USBDS_install((uint8_t*)Product_String_Descriptor, &(usbd_dev_desc->iProduct), &usbdc);
	// przygotowujemy SerialNumber_String_Descriptor
	SerialNumber_fill();
	USBDS_install((uint8_t*)SerialNumber_String_Descriptor, &(usbd_dev_desc->iSerialNumber), &usbdc);
}


static void configure_vcom(converter_params *vcom_params, CDC_VCOM_params *vcom, uint8_t usart_num) {
	// utworzenie buforow
	BUF_CreateOverExistingArea(&(vcom_params->buf.to_usart), vcom_params->space.to_usart, TO_USART_BUF_SIZE);
	BUF_CreateOverExistingArea(&(vcom_params->buf.to_vcom), vcom_params->space.to_vcom, TO_VCOM_BUF_SIZE);

	// wyczyszczenie buforow
	BUF_Clear(&(vcom_params->buf.to_usart));
	BUF_Clear(&(vcom_params->buf.to_vcom));

	// wypelnienie pola struktury vcom dedykowanego dla wyzszej klasy
	// wskaznikiem do struktury zawierajacej parametry danego portu
	// szeregowego oraz bufory - przyspiesza to funkcje wywolywane
	// przez biblioteke cdc_vcom
	vcom->upper_layer_data.pvoid = vcom_params;

	// zbindowanie funkcji i tablic cache z konkretnym objektem vcom
	CDC_VCOM_connect_change_params_handler(vcom, vCOM_change_params);
	CDC_VCOM_connect_RxD_handler(vcom, vCOM_Rx_handler);
	CDC_VCOM_connect_TxD_handler(vcom, vCOM_Tx_handler);
	CDC_VCOM_connect_RxD_buffer(vcom, vcom_params->cache.rx);
	CDC_VCOM_connect_TxD_buffer(vcom, vcom_params->cache.tx);

	vcom_params->usart_num = usart_num;
	if(usart_num == 1) {
		vcom_params->USARTx = USART1;
		conv_usart1 = vcom_params;
	}
	else if(usart_num == 2) {
		vcom_params->USARTx = USART2;
		conv_usart2 = vcom_params;
	}
	else if(usart_num == 3) {
		vcom_params->USARTx = USART3;
		conv_usart3 = vcom_params;
	}
}


// -----------------------------------------------------------------------------
//  usb_rs232_converter_init 
// -----------------------------------------------------------------------------
void converter_init(void) {
	usbd_init();


#if(USE_VCOM1)
	configure_vcom(&vcom1_params, &(vcom1_params.vcom), VCOM1_USART_NUM);
#endif
#if(USE_VCOM2)
	configure_vcom(&vcom2_params, &(vcom2_params.vcom), VCOM2_USART_NUM);
#endif


	USBD_activate(&usbdc);
}
