/*
================================================================================
 File: usbd_ccm.c
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       USBDevice configurations and classes manager
 Compiler:      ANSI C
 Hardware:
 Doc:
--------------------------------------------------------------------------------
 Includes:
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostępnione jest do celów poglądowych,
                nie jest przetestowane pod kątem bezpieczeństwa ani spełnienia
                jakichkolwiek norm wymaganych aby urządzenie używające go
                mogło być dopuszczone do użytku, dlatego używasz go
                na własną odpowiedzialność a jego autor nie ponosi żadnego
                typu odpowiedzialności za jakiekolwiek skutki jego użycia.
                Jeżeli nie zgadzasz się z powyższym warunkiem proszę
                o nieużywanie tego oprogramowania.
================================================================================
*/

#include <stdlib.h>
#include <string.h>


#ifndef USBD_CCM_H_
#include "usbd_ccm.h"
#endif






#if(USBD_CCM_DYNAMIC_MEMORY_SUPPORT == USBD_FEATURE_ON)
USBDC_params *CCM_create_and_init(void) {
	USBDC_params *usbdc = (USBDC_params *)malloc((size_t)sizeof(USBDC_params));
	if(usbdc == NULL) {
		return(0);
	}

	USBDC_init(usbdc);

	return(usbdc);
}
#endif








#if(USBD_CCM_DYNAMIC_MEMORY_SUPPORT == USBD_FEATURE_ON)
void CCM_destroy(USBDC_params *usbdc) {
	if(usbdc) {
		free((void*)usbdc);
	}
}
#endif








uint16_t CCM_get_desc_size(CCM_session_params *session) {
	uint16_t size = 0;

	if(session) {
		size = session->desc_size.uint16;
	}

	return(size);
}








#if(USBD_CCM_DYNAMIC_MEMORY_SUPPORT == USBD_FEATURE_ON)
uint8_t *CCM_create_desc(uint16_t desc_size, CCM_session_params* session) {
	uint8_t *desc;

	if(!session || !desc_size) {
		return(0);
	}
	if(!session->usbdc) {
		return(0);
	}

	desc = (uint8_t*)malloc((size_t)desc_size);

	if(desc == NULL) {
		return(0);
	}

	session->usbdc->configuration_descriptor = desc;

	memset((void*)desc, 0, (size_t)desc_size);

	return(desc);
}
#endif








#if(USBD_CCM_DYNAMIC_MEMORY_SUPPORT == USBD_FEATURE_ON)
void CCM_destroy_desc(CCM_session_params* session) {
	if(!session) {
		return;
	}
	if(!session->usbdc) {
		return;
	}
	if(!session->usbdc->configuration_descriptor) {
		return;
	}

	free((void*)(session->usbdc->configuration_descriptor));
}
#endif








void CCM_open_config(void *usbd, USBDC_params *usbdc, CCM_session_params* session) {
	if(!usbdc || !session) {
		return;
	}

	USBDC_bind_to_port(usbdc, usbd);

	memset((void*)session, 0, (size_t)sizeof(CCM_session_params));

	session->usbdc = usbdc;
	USBDC_init(usbdc);
	session->desc_size.uint16 = sizeof(USBD_configuration_descriptor);
}








uint8_t CCM_close_config(uint8_t *buf, CCM_session_params* session) {
	union {
		uint16_t uint16;
		uint8_t uint8;
	}counter;
	uint8_t result;
	USBD_configuration_descriptor *usbdc_desc = (USBD_configuration_descriptor *)buf;

	if(!buf || !session) {
		return(CCM_ERROR_BAD_PARAMS);
	}
	if(!session->usbdc) {
		return(CCM_ERROR_BAD_PARAMS);
	}

	session->usbdc->configuration_descriptor = buf;
	memset((void*)buf, 0, session->desc_size.uint16);

	CCM_adjust_endpoints(session->usbdc, session);

	usbdc_desc->bLength = 0x09;
	usbdc_desc->bDescriptorType = USBD_DESC_TYPE_CONFIGURATION;
	usbdc_desc->wTotalLength.L = session->desc_size.elements.L;
	usbdc_desc->wTotalLength.H = session->desc_size.elements.H;
	usbdc_desc->bNumInterfaces = session->num_reserved_interfaces.uint8;
	usbdc_desc->bmAttributes = 0x80;

	buf += sizeof(USBD_configuration_descriptor);

	for(counter.uint16 = 0; counter.uint16 < USBD_MAX_NUM_INTERFACES; counter.uint16++) {
		session->interface[counter.uint8].descriptor = buf;
		buf += session->interface[counter.uint8].desc_size;

		// clear slaves handlers - only masters can finish their configuration
		if(session->interface[counter.uint8].association.slave.master != counter.uint8) {
			session->interface[counter.uint8].class_finish_installation = 0;
		}
	}

	for(counter.uint16 = 0; counter.uint16 < USBD_MAX_NUM_INTERFACES; counter.uint16++) {
		if(session->interface[counter.uint8].class_finish_installation) {
			result = session->interface[counter.uint8].class_finish_installation(session->usbdc, counter.uint8, session);
			if(result) {
				return(result);
			}
		}
	}

	return(CCM_ERROR_NONE);
}








uint8_t CCM_close_and_install_config(uint8_t *buf, CCM_session_params* session) {
	uint8_t result = CCM_close_config(buf, session);
	if(result == CCM_ERROR_NONE) {
		buf[5] = USBDC_install (session->usbdc);
	}

	return(result);
}








uint8_t CCM_add_ep(CCM_ep_params_add *ep_params, CCM_session_params* session) {
	uint16_t mps;
	uint16_t mps_by_type;
	uint8_t interface;
	uint8_t ep_number;
	uint8_t result;

	if(!ep_params || !session) {
		return(CCM_ERROR_BAD_PARAMS);
	}

	if(session->num_reserved_endpoints >= USBD_MAX_NUM_ENDPOINTS) {
		return(CCM_ERROR_TOO_MANY_ENDPOINTS);
	}

	result = CCM_get_free_ep(session->usbdc, ep_params, session, &ep_number);

	if(!ep_number) {
		return(result);
	}

	mps = USBDEP_get_max_possible_packet_size(ep_number, session->usbdc);

	if(ep_params->type & USBDEP_TYPE_CONTROL) {
		mps_by_type = USBDEP_FS_CONTROL_MAX_POSSIBLE_PACKET_SIZE;
	}

	if(ep_params->type & USBDEP_TYPE_BULK) {
		mps_by_type = USBDEP_FS_BULK_MAX_POSSIBLE_PACKET_SIZE;
	}

	if(ep_params->type & USBDEP_TYPE_INTERRUPT) {
		mps_by_type = USBDEP_FS_INTERRUPT_MAX_POSSIBLE_PACKET_SIZE;
	}

	if(ep_params->type & USBDEP_TYPE_ISOCHRONOUS) {
		mps_by_type = USBDEP_FS_ISOCHRONOUS_MAX_POSSIBLE_PACKET_SIZE;
	}

	if(mps > mps_by_type) {
		mps = mps_by_type;
	}

	if(ep_params->max_packet_size.uint16 > mps) {
		ep_params->max_packet_size.uint16 = mps;
	}

	interface = ep_params->interface_num;
	if(!session->interface[interface].num_endpoints) {
		session->interface[interface].first_ep_num = ep_number;
		session->interface[interface].last_ep_num = ep_number;
	}
	else {
		session->endpoint[session->interface[interface].last_ep_num].next_ep_belong_to_this_interface = ep_number;
		session->interface[interface].last_ep_num = ep_number;
	}
	session->num_reserved_endpoints++;
	session->interface[interface].num_endpoints++;
	session->endpoint[ep_number].next_ep_belong_to_this_interface = 0;

	session->endpoint[ep_number].type = ep_params->type;
	session->endpoint[ep_number].ep_interface_identificator = ep_params->ep_id;
	session->endpoint[ep_number].max_packet_size = ep_params->max_packet_size.uint16;
	session->endpoint[ep_number].max_packet_size_priority = ep_params->max_packet_size_priority;

	return(CCM_ERROR_NONE);
}








uint16_t CCM_add_interface(void *data, CCM_finish_installation finish_func, uint16_t desc_size, CCM_session_params* session) {
	uint8_t interface;

	if(!data || !finish_func || !desc_size || !session) {
		return(CCM_ERROR_BAD_PARAMS << 8);
	}

	if(session->num_reserved_interfaces.uint16 >= USBD_MAX_NUM_INTERFACES) {
		return(CCM_ERROR_TOO_MANY_INTERFACES << 8);
	}

	session->interface[session->num_reserved_interfaces.uint8].data = data;
	session->interface[session->num_reserved_interfaces.uint8].class_finish_installation = finish_func;
	session->interface[session->num_reserved_interfaces.uint8].desc_size = desc_size;
	session->interface[session->num_reserved_interfaces.uint8].first_ep_num = 0;
	session->interface[session->num_reserved_interfaces.uint8].last_ep_num = 0;
	session->interface[session->num_reserved_interfaces.uint8].num_endpoints = 0;
	session->interface[session->num_reserved_interfaces.uint8].descriptor = 0;

	session->interface[session->num_reserved_interfaces.uint8].association.slave.master = session->num_reserved_interfaces.uint8;
	session->interface[session->num_reserved_interfaces.uint8].association.slave.previous_associated_to_master = session->num_reserved_interfaces.uint8;
	session->interface[session->num_reserved_interfaces.uint8].association.slave.next_associated_to_master = session->num_reserved_interfaces.uint8;

	// slaves settings are set to 0 because we always add master before slaves
	// so this interface has no any slaves yet
	session->interface[session->num_reserved_interfaces.uint8].association.master.num_slaves = 0;
	session->interface[session->num_reserved_interfaces.uint8].association.master.first_slave = 0;
	session->interface[session->num_reserved_interfaces.uint8].association.master.last_slave = 0;

	interface = session->num_reserved_interfaces.uint8;

	session->num_reserved_interfaces.uint16++;
	session->desc_size.uint16 += desc_size;

	return((uint16_t)(interface));
}








uint8_t CCM_associate_interface(uint8_t slave, uint8_t master, uint8_t slave_id, CCM_session_params* session) {
	uint8_t interface;

	if(!session) {
		return(CCM_ERROR_BAD_PARAMS);
	}

	if((session->num_reserved_interfaces.uint16 <= master)
			|| (session->num_reserved_interfaces.uint16 <= slave)) {
		return(CCM_ERROR_BAD_PARAMS);
	}

	session->interface[slave].association.slave.master = master;
	session->interface[slave].association.slave.previous_associated_to_master = session->interface[master].association.master.last_slave;
	session->interface[slave].association.slave.next_associated_to_master = slave;
	session->interface[slave].association.slave.slave_id = slave_id;

	// master and previous slave changes
	if(session->interface[master].association.master.num_slaves) {
		interface = session->interface[master].association.master.last_slave;
		session->interface[interface].association.slave.next_associated_to_master = slave;
		session->interface[master].association.master.last_slave = slave;
		session->interface[master].association.master.num_slaves++;
	}
	else {
		session->interface[master].association.master.num_slaves = 1;
		session->interface[master].association.master.first_slave = slave;
		session->interface[master].association.master.last_slave = slave;
	}

	return(CCM_ERROR_NONE);
}








uint8_t CCM_remove_last_interface(CCM_session_params* session) {
	union {
		uint8_t uint8;
		uint16_t uint16;
	}interface;

	if(!session) {
		return(CCM_ERROR_BAD_PARAMS);
	}

	if(!(session->num_reserved_interfaces.uint16)) {
		return (CCM_ERROR_NONE);
	}

	interface.uint16 = session->num_reserved_interfaces.uint16 - 1;

	if(session->interface[interface.uint8].num_endpoints) {
		CCM_interface_remove_endpoints(interface.uint8, session);
	}

	session->num_reserved_interfaces.uint16 = interface.uint16;

	return(CCM_ERROR_NONE);
}








uint8_t CCM_interface_remove_endpoints(uint8_t interface, CCM_session_params* session) {
	uint8_t ep_num;
	uint8_t num_endpoints;

	if(!session) {
		return(CCM_ERROR_BAD_PARAMS);
	}

	num_endpoints = session->interface[interface].num_endpoints;
	ep_num = session->interface[interface].first_ep_num;

	session->num_reserved_endpoints -= num_endpoints;

	for(; num_endpoints; num_endpoints--) {
		ep_num = session->endpoint[ep_num].next_ep_belong_to_this_interface;
		session->ep_state &= ~((uint16_t)(0x1 << ep_num));
	}

	session->interface[interface].num_endpoints = 0;

	return(CCM_ERROR_NONE);
}








uint8_t CCM_FINISH_get_ep_params(CCM_ep_params_get *ep_params, uint8_t ep_index, uint8_t interface, CCM_session_params* session) {
	uint8_t ep_number;

	if(!ep_params || !session) {
		return(CCM_ERROR_BAD_PARAMS);
	}

	if(ep_index >= session->interface[interface].num_endpoints) {
		return(CCM_ERROR_BAD_PARAMS);
	}

	ep_number = session->interface[interface].first_ep_num;

	for(; ep_index; ep_index--) {
		ep_number = session->endpoint[ep_number].next_ep_belong_to_this_interface;
	}

	if(!ep_number) {
		return(CCM_ERROR_BAD_PARAMS);
	}

	ep_params->type = session->endpoint[ep_number].type;
	ep_params->max_packet_size.uint16 = session->endpoint[ep_number].max_packet_size;
	ep_params->ep_id = session->endpoint[ep_number].ep_interface_identificator;
	ep_params->ep_num = ep_number;

	return(CCM_ERROR_NONE);
}








uint8_t CCM_FINISH_get_interface_params(CCM_interface_params_get *interface_params, uint8_t interface, CCM_session_params* session) {
	if(!interface_params || !session) {
		return(CCM_ERROR_BAD_PARAMS);
	}

	if(interface >= session->num_reserved_interfaces.uint16) {
		return(CCM_ERROR_BAD_PARAMS);
	}

	interface_params->data = session->interface[interface].data;
	interface_params->num_endpoints = session->interface[interface].num_endpoints;
	interface_params->desc_size = session->interface[interface].desc_size;
	interface_params->association.association = 0;
	interface_params->descriptor = session->interface[interface].descriptor;
	if(session->interface[interface].association.slave.master != interface) {
		interface_params->association.association = 0x55;
		interface_params->association.slave.associated_master = session->interface[interface].association.slave.master;
		interface_params->association.slave.slave_id = session->interface[interface].association.slave.slave_id;
	}
	else {
		interface_params->association.slave.associated_master = interface;
		interface_params->association.slave.slave_id = 0;
	}
	if(session->interface[interface].association.master.num_slaves) {
		interface_params->association.association = 0x55;
		interface_params->association.master.num_slaves = session->interface[interface].association.master.num_slaves;
	}
	else {
		interface_params->association.master.num_slaves = 0;
	}

	return(CCM_ERROR_NONE);
}








uint8_t CCM_FINISH_get_slave_interface_num_by_id(uint8_t *interface_slave, uint8_t master_interface, uint8_t slave_id, CCM_session_params* session) {
	uint8_t interface;

	if(!interface_slave || !session) {
		return(CCM_ERROR_BAD_PARAMS);
	}

	if(master_interface >= session->num_reserved_interfaces.uint16) {
		return(CCM_ERROR_BAD_PARAMS);
	}

	if(session->interface[master_interface].association.master.num_slaves <= slave_id) {
		return(CCM_ERROR_BAD_PARAMS);
	}

	interface = session->interface[master_interface].association.master.first_slave;
	for(; slave_id; slave_id--) {
		interface = session->interface[interface].association.slave.next_associated_to_master;
	}

	*interface_slave = interface;

	return(CCM_ERROR_NONE);
}
