/*
================================================================================
 File: usbd_ccm.h
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       USBDevice configurations and classes manager
 Compiler:      ANSI C
 Hardware:
 Doc:
--------------------------------------------------------------------------------
 Includes:
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostępnione jest do celów poglądowych,
                nie jest przetestowane pod kątem bezpieczeństwa ani spełnienia
                jakichkolwiek norm wymaganych aby urządzenie używające go
                mogło być dopuszczone do użytku, dlatego używasz go
                na własną odpowiedzialność a jego autor nie ponosi żadnego
                typu odpowiedzialności za jakiekolwiek skutki jego użycia.
                Jeżeli nie zgadzasz się z powyższym warunkiem proszę
                o nieużywanie tego oprogramowania.
================================================================================
*/

#ifndef USBD_CCM_H_
#define USBD_CCM_H_

#ifndef USBD_H_
#include "usbd.h"
#endif


#ifndef USBD_CCM_DYNAMIC_MEMORY_SUPPORT
#define USBD_CCM_DYNAMIC_MEMORY_SUPPORT			USBD_FEATURE_OFF
#endif




typedef enum {
			CCM_ERROR_NONE = 0,
			CCM_ERROR_BAD_PARAMS,
			CCM_ERROR_TOO_MANY_ENDPOINTS,
			CCM_ERROR_TOO_MANY_ENDPOINTS_WITH_SPECIFIC_TYPE,
			CCM_ERROR_TOO_MANY_INTERFACES,
			CCM_ERROR_TOO_BIG_EP_MAX_PACKET_SIZE,
			CCM_ERROR_TOO_BIG_EP_BUFFERS_SPACE_SUM,
			CCM_ERROR_TOO_BIG_EP_MANDATORY_BUFFERS_SPACE_SUM,
			CCM_FINISH_ERROR
}CCM_error_types;




struct _CCM_session_params {
	USBDC_params *usbdc;

	union {
		uint16_t uint16;

		struct {
			uint8_t L;
			uint8_t H;
		}elements;
	}desc_size;

	uint8_t num_reserved_endpoints;

	uint16_t ep_state;

	union {
		uint16_t uint16;
		uint8_t uint8;
	}num_reserved_interfaces;

	struct {
		void *data;

		uint8_t (*class_finish_installation) (USBDC_params*, uint8_t, struct _CCM_session_params*);

		uint8_t *descriptor;

		uint16_t desc_size;

		uint8_t num_endpoints;

		uint8_t first_ep_num;

		uint8_t last_ep_num;

		struct {

			struct {
				uint8_t num_slaves;

				uint8_t first_slave;

				uint8_t last_slave;
			}master;

			struct {
				uint8_t master;

				uint8_t slave_id;

				uint8_t previous_associated_to_master;

				uint8_t next_associated_to_master;
			}slave;

		}association;

	}interface[USBD_MAX_NUM_INTERFACES];

	struct {
		uint16_t max_packet_size;

		// 0 - 255
		// 0 - max packet size fully scalable when needed
		// 255 - max packet size not modifiable
		uint8_t max_packet_size_priority;

		uint8_t type;

		uint8_t ep_interface_identificator;

		uint8_t next_ep_belong_to_this_interface;
	}endpoint[USBD_MAX_NUM_ENDPOINTS];
};
typedef struct _CCM_session_params CCM_session_params;



typedef struct {
	uint8_t type;
	// 0 - 255
	// 0 - max packet size fully scalable when needed
	// 255 - max packet size not modifiable
	uint8_t max_packet_size_priority;
	union {
		uint16_t uint16;
		struct {
			uint8_t L;
			uint8_t H;
		}uint8;
	}max_packet_size;
	uint8_t ep_id;
	uint8_t interface_num;
}CCM_ep_params_add;



typedef struct {
	uint8_t type;
	uint8_t ep_num;
	union {
		uint16_t uint16;
		struct {
			uint8_t L;
			uint8_t H;
		}uint8;
	}max_packet_size;
	uint8_t ep_id;
}CCM_ep_params_get;



typedef struct {
	void *data;
	uint8_t *descriptor;
	uint16_t desc_size;
	uint8_t num_endpoints;

	struct {
		uint8_t association;

		struct {
			uint8_t num_slaves;
		}master;

		struct {
			uint8_t associated_master;
			uint8_t slave_id;
		}slave;
	}association;
}CCM_interface_params_get;



typedef uint8_t (*CCM_finish_installation) (USBDC_params*, uint8_t, CCM_session_params*);



/**-----------------------------------------------------------------
 * Tworzenie konfiguracji
 */

/**
 * Funkcja tworzaca dynamicznie konfiguracje oraz inicjalizujaca ja
 */
#if(USBD_CCM_DYNAMIC_MEMORY_SUPPORT == USBD_FEATURE_ON)
USBDC_params *CCM_create_and_init(void);
#endif

/**
 * Funkcja niszczaca stworzona wczesniej dynamicznie konfiguracje
 */
#if(USBD_CCM_DYNAMIC_MEMORY_SUPPORT == USBD_FEATURE_ON)
void CCM_destroy(USBDC_params *usbdc);
#endif




/**-----------------------------------------------------------------
 * Pobieranie przewidywanego rozmiaru deskryptora konfiguracji
 */

/**
 * Funkcja pobierajaca wymagany rozmiar deskryptora konfiguracji
 */
uint16_t CCM_get_desc_size(CCM_session_params *session);




/**-----------------------------------------------------------------
 * Tworzenie deskryptora konfiguracji
 */

/**
 * Funkcja alokujaca dynamicznie pamiec dla deskryptora konfiguracji.
 */
#if(USBD_CCM_DYNAMIC_MEMORY_SUPPORT == USBD_FEATURE_ON)
uint8_t *CCM_create_desc(uint16_t desc_size, CCM_session_params* session);
#endif

/**
 * Funkcja niszczaca deskryptor konfiguracji, ktory zostal stworzony dynamicznie
 */
#if(USBD_CCM_DYNAMIC_MEMORY_SUPPORT == USBD_FEATURE_ON)
void CCM_destroy_desc(CCM_session_params* session);
#endif




/**-----------------------------------------------------------------
 * Podlaczanie klas do konfiguracji
 */

/**
 * Funkcja otwierajaca sesje ccm. Po wywolaniu tej funkcji
 * mozna podpinac za pomoca bibliotek klas rozne interfejsy
 * oraz zespoly interfejsow do danej konfiguracji
 */
void CCM_open_config(void *usbd, USBDC_params *usbdc, CCM_session_params* session);

/**
 * Funkcja zamykajaca sesje ccm
 * Po jej wywolaniu mozna zainstalowac dana konfiguracje w porcie.
 */
uint8_t CCM_close_config(uint8_t *buf, CCM_session_params* session);

/**
 * Funkcja zamykajaca sesje ccm i instalujaca konfiguracje w porcie usbd
 */
uint8_t CCM_close_and_install_config(uint8_t *buf, CCM_session_params* session);




/**
 *  Funkcja zwraca pole bitowe kompatybilne z USBDEP_types.
 *  Wartosc ta okresla, jakie typy transmisji wspiera dany endpoint
 *
 *  \param ep_number numer Endpointa ktorego parametry
 *  chcemy uzyskac
 *  \param usbdc wskaznik do konfiguracji wywolujacej ta funkcje
 *  \return pole bitowe ze wspieranymi typami transmisji
 */
uint8_t CCM_get_ep_supported_types(uint8_t ep_number, USBDC_params *usbdc);

uint8_t CCM_get_free_ep(USBDC_params *usbdc, CCM_ep_params_add *ep_params, CCM_session_params* session, uint8_t *ep_index);

uint8_t CCM_adjust_endpoints(USBDC_params *usbdc, CCM_session_params* session);


uint8_t CCM_add_ep(CCM_ep_params_add *ep_params, CCM_session_params* session);

uint16_t CCM_add_interface(void *data, CCM_finish_installation finish_func, uint16_t desc_size, CCM_session_params* session);

uint8_t  CCM_associate_interface(uint8_t slave, uint8_t master, uint8_t slave_id, CCM_session_params* session);

uint8_t  CCM_remove_last_interface(CCM_session_params* session);

uint8_t  CCM_interface_remove_endpoints(uint8_t interface, CCM_session_params* session);


uint8_t CCM_FINISH_get_ep_params(CCM_ep_params_get *ep_params, uint8_t ep_index, uint8_t interface, CCM_session_params* session);

uint8_t CCM_FINISH_get_interface_params(CCM_interface_params_get *interface_params, uint8_t interface, CCM_session_params* session);

uint8_t CCM_FINISH_get_slave_interface_num_by_id(uint8_t *interface_slave, uint8_t master_interface, uint8_t slave_id, CCM_session_params* session);


#endif /* USBD_CCM_H_ */
